﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class DepartmentRepository : IDepartmentRepository
    {
        #region Private

        private const string RepositoryName = "DepartmentRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetByParentIDSignature = "GetByParentID";
        private const string GetByParentIDAndRootIDSignature = "GetByParentIDAndRootID";
        private const string GetByRootIDSignature = "GetByRootID";
        private const string GetByRootIDLeafOnlySignature = "GetByRootIDLeafOnly";
        private const string GetByRootIDNotLeafSignature = "GetByRootIDNotLeaf";
        private const string GetByRootIDNoProductSignature = "GetByRootIDNoProduct";
        private const string GetRootDepartmentSignature = "GetRootDepartment";
        private const string GetAllSignature = "GetAll";
        private const string GetAllWithoutRootDepartmentSignature = "GetAllWithoutRootDepartment";
        private const string GetAllLeafDepartmentSignature = "GetAllLeafDepartment";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetByParentUrlNameSignature = "GetByParentUrlName";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string IsLeafSignature = "IsLeaf";
        private const string IsNotLeftSignature = "IsNotLeaf";
        private const string IsUrlNameNotLeafSignature = "IsUrlNameNotLeaf";
        private const string HasDepartmentInDiscountGroupSignature = "HasDepartmentInDescountGroup";
        private const string SearchDepartmentSignature = "SearchDepartment";
        private const string SearchDepartmentByRootIDSignature = "SearchDepartmentByRootID";
        private const string SearchRootDepartmentSignature = "SerachRootDepartment";
        private const string GetLeafFromDepartmentIDSignature = "GetLeafFromDepartmentID";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "DepartmentLocale", "Department" } );

        private DepartmentRepositoryNoCache _repository = new DepartmentRepositoryNoCache();

        #endregion

        #region Constructors

        public DepartmentRepository()
        {
        }

        public DepartmentRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Members

        public Department Save( Department department )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Department newDepartment = _repository.Save( department );
            _dataCache.ClearRepositoryCache();
            return newDepartment;
        }

        public Department SaveWithIdentity( Department department )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Department newDepartment = _repository.SaveWithIdentity( department );
            _dataCache.ClearRepositoryCache();
            return newDepartment;
        }

        public void Delete( string departmentID )
        {
            _repository.Delete( departmentID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public void ReArrangeSortOrder( Culture culture, string parentDepartmentID, string departmentID, BoolFilter isEnabled )
        {
            _repository.ReArrangeSortOrder( culture, parentDepartmentID, departmentID, isEnabled );
            _dataCache.ClearRepositoryCache();
        }

        public void ReArrangeSortOrderByParentDepartmentID( Culture culture, string parentDepartmentID, BoolFilter isEnabled )
        {
            _repository.ReArrangeSortOrderByParentDepartmentID( culture, parentDepartmentID, isEnabled );
            _dataCache.ClearRepositoryCache();
        }

        public void UpdateSortOrder( string[] sortedDepartmentIDs )
        {
            _repository.UpdateSortOrder( sortedDepartmentIDs );
            _dataCache.ClearRepositoryCache();
        }

        public int GetCountAllWithoutRootDepartment()
        {
            return _repository.GetCountAllWithoutRootDepartment();
        }

        public IList<Department> GetByParentID(
            Culture culture,
            string parentDepartmentID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByParentIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentDepartmentID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByParentID( culture, parentDepartmentID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByParentIDSignature,
                    list,
                    culture.CultureID,
                    parentDepartmentID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByParentIDAndRootID(
           Culture culture,
           string parentDepartmentID,
           string rootID,
           string sortBy,
           BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByParentIDAndRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentDepartmentID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByParentIDAndRootID( culture, parentDepartmentID, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByParentIDAndRootIDSignature,
                    list,
                    culture.CultureID,
                    parentDepartmentID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByParentID(
           Culture culture,
           string parentDepartmentID,
           string sortBy,
           BoolFilter isEnabled,
           int startIndex,
           int endIndex,
           out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue( GetByParentIDSignature,
                _cacheDependency,
                culture.CultureID,
                parentDepartmentID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByParentID(
                    culture, parentDepartmentID, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetByParentIDSignature,
                    list,
                    culture.CultureID,
                    parentDepartmentID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Department>) list.First;
        }

        public IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootID( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByRootIDLeafOnly(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByRootIDLeafOnlySignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDLeafOnly( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDLeafOnlySignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByRootIDNotLeaf(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByRootIDNotLeafSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDNotLeaf( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDNotLeafSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByRootIDNoProduct(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetByRootIDNoProductSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetByRootIDNoProduct( culture, rootID, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetByRootIDNoProductSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetByRootID(
            Culture culture,
            string rootID,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetByRootIDSignature,
                _cacheDependency,
                culture.CultureID,
                rootID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByRootID(
                    culture, rootID, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetByRootIDSignature,
                    list,
                    culture.CultureID,
                    rootID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Department>) list.First;
        }

        public IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue(
                GetRootDepartmentSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled );

            if (list == null)
            {
                list = _repository.GetRootDepartment( culture, sortBy, isEnabled );

                _dataCache.SetValue(
                    GetRootDepartmentSignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled );
            }

            return list;
        }

        public IList<Department> GetRootDepartment(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetRootDepartmentSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                isEnabled,
                startIndex,
                endIndex );

            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetRootDepartment(
                    culture, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue(
                    GetRootDepartmentSignature,
                    list,
                    culture.CultureID,
                    sortBy,
                    isEnabled,
                    startIndex,
                    endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (List<Department>) list.First;
        }

        public IList<Department> GetByParentUrlName(
             Culture culture,
             string parentDepartmentName,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue(
                GetByParentUrlNameSignature, _cacheDependency, culture.CultureID, parentDepartmentName, sortBy, isEnabled, startIndex, endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetByParentUrlName( culture, parentDepartmentName, sortBy, isEnabled, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue( GetByParentUrlNameSignature, list, culture.CultureID, parentDepartmentName, sortBy, isEnabled, startIndex, endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Department>) list.First;
        }

        public Department GetOne( Culture culture, string departmentID )
        {
            Department department = (Department) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, departmentID );

            if (department == null)
            {
                department = _repository.GetOne( culture, departmentID );
                _dataCache.SetValue( GetOneSignature, department, culture.CultureID, departmentID );
            }

            return department;
        }

        public Department GetOneByUrlName( Culture culture, string urlName )
        {
            Department department = (Department) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, urlName );

            if (department == null)
            {
                department = _repository.GetOneByUrlName( culture, urlName );
                _dataCache.SetValue( GetOneSignature, department, culture.CultureID, urlName );
            }

            return department;
        }

        public IList<Department> GetAll( Culture culture, string sortBy )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy );

                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<Department> GetAllLeafDepartment( Culture culture, string sortBy )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( GetAllLeafDepartmentSignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAllLeafDepartment( culture, sortBy );

                _dataCache.SetValue( GetAllLeafDepartmentSignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<Department> GetAll( Culture culture, string sortBy, BoolFilter isEnabled )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy, isEnabled.ToString() );

            if (list == null)
            {
                list = _repository.GetAll( culture, sortBy, isEnabled );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy, isEnabled.ToString() );
            }

            return (IList<Department>) list;
        }

        public IList<Department> GetAll(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            Pair list = (Pair) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, sortBy, startIndex, endIndex );
            if (list == null)
            {
                list = new Pair();
                list.First = _repository.GetAll( culture, sortBy, startIndex, endIndex, out howManyItems );
                list.Second = howManyItems;
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, sortBy, startIndex, endIndex );
            }
            howManyItems = ConvertUtilities.ToInt32( list.Second );
            return (IList<Department>) list.First;
        }

        public IList<Department> GetAllWithoutRootDepartment( Culture culture, string sortBy )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( GetAllWithoutRootDepartmentSignature, _cacheDependency, culture.CultureID, sortBy );

            if (list == null)
            {
                list = _repository.GetAllWithoutRootDepartment( culture, sortBy );

                _dataCache.SetValue( GetAllWithoutRootDepartmentSignature, list, culture.CultureID, sortBy );
            }

            return list;
        }

        public IList<ILocale> GetLocales( string departmentID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, departmentID );

            if (list == null)
            {
                list = _repository.GetLocales( departmentID );

                _dataCache.SetValue( GetLocalesSignature, list, departmentID );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );
            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }
            return list;
        }

        public bool IsLeaf( string departmentID )
        {
            object result = _dataCache.GetValue( IsLeafSignature, _cacheDependency, departmentID );

            if (result == null)
            {
                result = _repository.IsLeaf( departmentID );
                _dataCache.SetValue( IsLeafSignature, result, departmentID );
            }

            return (bool) result;
        }

        public bool IsDepartmentIDNotLeaf( string parentDepartmentID )
        {
            object result = _dataCache.GetValue( IsNotLeftSignature, _cacheDependency, parentDepartmentID );
            if (result == null)
            {
                result = _repository.IsDepartmentIDNotLeaf( parentDepartmentID );
                _dataCache.SetValue( IsNotLeftSignature, result, parentDepartmentID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public bool IsUrlNameNotLeaf( string parentDepartmentUrlName )
        {
            object result = _dataCache.GetValue( IsUrlNameNotLeafSignature, _cacheDependency, parentDepartmentUrlName );
            if (result == null)
            {
                result = _repository.IsUrlNameNotLeaf( parentDepartmentUrlName );
                _dataCache.SetValue( IsUrlNameNotLeafSignature, result, parentDepartmentUrlName );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public void RefreshUrlName( string departmentID, string name )
        {
            _repository.RefreshUrlName( departmentID, name );
            _dataCache.ClearRepositoryCache();
        }

        public bool HasDepartmentInDiscountGroup( string discountgroupID )
        {
            object result = _dataCache.GetValue( HasDepartmentInDiscountGroupSignature, _cacheDependency, discountgroupID );
            if (result == null)
            {
                result = _repository.HasDepartmentInDiscountGroup( discountgroupID );
                _dataCache.SetValue( HasDepartmentInDiscountGroupSignature, result, discountgroupID );
            }
            return ConvertUtilities.ToBoolean( result );
        }

        public IList<Department> SearchDepartment(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( SearchDepartmentSignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString() );
            if (list == null)
            {
                list = _repository.SearchDepartment( culture, sortBy, searchFilter );
                _dataCache.SetValue( SearchDepartmentSignature, list, culture.CultureID, sortBy, searchFilter.ToString() );
            }
            return (IList<Department>) list;
        }

        public IList<Department> SearchDepartmentByRootID(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            string rootID
            )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( SearchDepartmentByRootIDSignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString(), rootID );
            if (list == null)
            {
                list = _repository.SearchDepartmentByRootID( culture, sortBy, searchFilter, rootID );
                _dataCache.SetValue( SearchDepartmentByRootIDSignature, list, culture.CultureID, sortBy, searchFilter.ToString(), rootID );
            }
            return (IList<Department>) list;
        }

        public IList<Department> SearchRootDepartment(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter
            )
        {
            IList<Department> list = (IList<Department>) _dataCache.GetValue( SearchRootDepartmentSignature, _cacheDependency, culture.CultureID, sortBy, searchFilter.ToString() );
            if (list == null)
            {
                list = _repository.SearchRootDepartment( culture, sortBy, searchFilter );
                _dataCache.SetValue( SearchRootDepartmentSignature, list, culture.CultureID, sortBy, searchFilter.ToString() );
            }
            return (IList<Department>) list;
        }

        private string GenerateUniqueStringFromList( IList<string> list )
        {
            List<string> newList = new List<string>( list );
            return string.Join( ",", newList.ToArray() );
        }

        public IList<string> GetLeafFromDepartmentID(
            string parentDepartmentID, IList<string> list )
        {
            IList<string> t_list = (IList<string>) _dataCache.GetValue(
                GetLeafFromDepartmentIDSignature,
                _cacheDependency,
                parentDepartmentID,
                GenerateUniqueStringFromList( list ) );
            if (t_list == null)
            {
                t_list = _repository.GetLeafFromDepartmentID( parentDepartmentID, list );
                _dataCache.SetValue(
                    GetLeafFromDepartmentIDSignature,
                    t_list,
                    parentDepartmentID,
                    GenerateUniqueStringFromList( list ) );
            }
            return t_list;
        }

        #endregion
    }
}
