using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Domain.Stores;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class NewsLetterRepositoryNoCache : INewsLetterRepository
    {
        #region Private

        private const string allFields = " NewsLetter.Email, EmailHash, JoinDate, NewsLetter.StoreID ";

        private string ConvertSpecialFields( string columnName )
        {
            if (columnName.ToLower() == "email")
                return String.Format( "NewsLetter.{0}", columnName );
            else
                return columnName;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.ToLower().StartsWith( "email" ))
                return "ORDER BY NewsLetter." + sortBy;
            else
                return "ORDER BY " + sortBy;
        }

        private NewsLetter CreateNewsLetter( NewsLetter newsLetter )
        {
            DataAccess.ExecuteNonQuery(
                "INSERT INTO Newsletter(Email, EmailHash, JoinDate, StoreID) VALUES (" +
                "@Email, @EmailHash, @JoinDate, @StoreID);",
                DataAccess.CreateParameterString( newsLetter.Email ),
                DataAccess.CreateParameterString( newsLetter.EmailHash ),
                DataAccess.CreateParameterDateTime( newsLetter.JoinDate ),
                DataAccess.CreateParameterString( newsLetter.StoreID )
                );

            return newsLetter;
        }


        private NewsLetter UpdateNewsLetter( NewsLetter newsLetter, string newEmail, string newStoreID)
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Newsletter " +
                "SET Email = @EmailNew, " +
                "EmailHash = @EmailHash, " +
                "JoinDate = @JoinDate, " +
                "StoreID = @StoreID " +
                "WHERE Email = @EmailOld AND StoreID = @NewStoreID;",
                DataAccess.CreateParameterString( newEmail ),
                DataAccess.CreateParameterString( newsLetter.EmailHash ),
                DataAccess.CreateParameterDateTime( newsLetter.JoinDate ),
                DataAccess.CreateParameterString( newStoreID ),
                DataAccess.CreateParameterString( newsLetter.Email ),
                DataAccess.CreateParameterString( newsLetter.StoreID )
                );

            newsLetter.Email = newEmail;
            newsLetter.StoreID = newStoreID;
            return newsLetter;

        }

        private NewsLetter CreateOneFromRow( DataRow row )
        {
            NewsLetter newsLetter = new NewsLetter();

            newsLetter.Email = row["Email"].ToString();
            newsLetter.EmailHash = row["EmailHash"].ToString();
            newsLetter.JoinDate = ConvertUtilities.ToDateTime( row["JoinDate"] );
            newsLetter.StoreID = row["StoreID"].ToString();

            if (row.Table.Columns.Contains( "UserName" ))
                newsLetter.UserName = row["UserName"].ToString();

            return newsLetter;
        }

        private IList<NewsLetter> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<NewsLetter> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<NewsLetter> list = new List<NewsLetter>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public NewsLetter Create( NewsLetter newsLetter )
        {
            return CreateNewsLetter( newsLetter );
        }

        public NewsLetter Update( NewsLetter newsLetter, string newEmail, string newStoreID )
        {
            return UpdateNewsLetter( newsLetter, newEmail, newStoreID );
        }

        public bool DeleteEmail( string emailHash, Store store )
        {
            int result = DataAccess.ExecuteNonQuery(
                "DELETE FROM Newsletter " +
                "WHERE EmailHash = @EmailHash AND StoreID = @StoreID; ",
                DataAccess.CreateParameterString( emailHash ), DataAccess.CreateParameterString( store.StoreID ) );

            if (result != 1)
                return false;
            else
                return true;
        }

        public bool DeleteEmailNoHash( string emailName, Store store )
        {
            int result = DataAccess.ExecuteNonQuery(
                "DELETE FROM Newsletter " +
                "WHERE Email = @Email AND StoreID = @StoreID; ",
                DataAccess.CreateParameterString( emailName ), DataAccess.CreateParameterString( store.StoreID ) );

            if (result != 1)
                return false;
            else
                return true;
        }

        public bool DeleteEmailByStore( Store store )
        {
            int result = DataAccess.ExecuteNonQuery(
                "DELETE FROM Newsletter " +
                "WHERE StoreID = @StoreID; ",
                DataAccess.CreateParameterString( store.StoreID ) );

            if (result != 1)
                return false;
            else
                return true;
        }

        public NewsLetter GetOne( string email, Store store )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                "FROM Newsletter " +
                "WHERE Email = @Email AND StoreID = @StoreID;",
                DataAccess.CreateParameterString( email ), DataAccess.CreateParameterString( store.StoreID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return NewsLetter.Null;
        }

        public IList<NewsLetter> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                " FROM Newsletter INNER JOIN Store ON Newsletter.StoreID = Store.StoreID " +
                 CreateSortByText( sortBy ) + "; " );

            return ConvertTableToList( table );
        }

        public IList<NewsLetter> GetAllByStore( Store store )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + allFields +
                " FROM Newsletter " +
                "WHERE StoreID = @StoreID;",
                DataAccess.CreateParameterString( store.StoreID ) );

            return ConvertTableToList( table );
        }

        public IList<NewsLetter> SearchNewsletter(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, ConvertSpecialFields( searchFilter.FieldName ), parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Newsletter.Email, Newsletter.EmailHash, Newsletter.JoinDate, Store.StoreName,Store.StoreID, Customer.UserName FROM Store " +
                "INNER JOIN Newsletter ON Store.StoreID = Newsletter.StoreID   " +
                "LEFT OUTER JOIN Customer ON Newsletter.Email = Customer.Email " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["NewsLetter.Email"], table.Columns["Customer.UserName"], table.Columns["Store.StoreName"] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT Email, UserName, StoreName FROM Customer, Store " +
                "WHERE 0 = 1; " );

            DataColumn[] keys = new DataColumn[] { table.Columns["Email"] };
            table.PrimaryKey = keys;

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
