using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Data;

namespace Vevo.Data
{
    public class StateRepositoryNoCache : IStateRepository
    {
        #region Private

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy + " ";
        }


        private State CreateState( State state )
        {
            DataAccess.ExecuteScalar(
                "INSERT INTO State( CountryCode, " +
                "StateCode, StateName, SortOrder, Enabled ) " +
                "VALUES ( @CountryCode, @StateCode, @StateName, @SortOrder, @Enabled );" +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterString( state.CountryCode ),
                DataAccess.CreateParameterString( state.StateCode ),
                DataAccess.CreateParameterString( state.StateName ),
                DataAccess.CreateParameterInt32( state.SortOrder ),
                DataAccess.CreateParameterBool( state.Enabled ) );
            return state;
        }

        private State UpdateState( State state, string newStateCode )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE State SET StateCode = @StateCodeNew, " +
                "SortOrder = @SortOrder, " +
                "StateName = @StateName, Enabled = @Enabled " +
                "WHERE CountryCode = @CountryCode AND StateCode = @StateCodeOld;",
                DataAccess.CreateParameterString( newStateCode ),
                DataAccess.CreateParameterInt32( state.SortOrder ),
                DataAccess.CreateParameterString( state.StateName ),
                DataAccess.CreateParameterBool( state.Enabled ),
                DataAccess.CreateParameterString( state.CountryCode ),
                DataAccess.CreateParameterString( state.StateCode ) );

            state.StateCode = newStateCode;
            return state;
        }

        private State CreateOneFromRow( DataRow row )
        {
            State state = new State();
            state.CountryCode = row["CountryCode"].ToString();
            state.StateCode = row["StateCode"].ToString();
            state.StateName = row["StateName"].ToString();
            state.SortOrder = Convert.ToInt32( row["SortOrder"] );
            state.Enabled = Convert.ToBoolean( row["Enabled"] );

            return state;
        }

        private IList<State> ConvertTableToList( DataTable table )
        {
            IList<State> list = new List<State>();

            for (int i = 0; i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public State Create( State state )
        {
            return CreateState( state );
        }

        public State Update( State state )
        {
            return UpdateState( state, state.StateCode );
        }

        public State Update( State state, string newStateCode )
        {
            return UpdateState( state, newStateCode );
        }

        public State GetOne( string countryCode, string stateCode )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SortOrder, CountryCode, StateCode, StateName, Enabled " +
                "FROM State " +
                "WHERE StateCode = @StateCode AND CountryCode = @CountryCode; ",
                DataAccess.CreateParameterString( stateCode ),
                DataAccess.CreateParameterString( countryCode ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return State.Null;
        }

        public void Delete( string countryCode, string stateCode )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM State WHERE StateCode = @StateCode AND CountryCode = @CountryCode;",
               DataAccess.CreateParameterString( stateCode ),
               DataAccess.CreateParameterString( countryCode ) );
        }

        public void DeleteByCountry( string countryCode )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM State WHERE CountryCode = @CountryCode;",
                DataAccess.CreateParameterString( countryCode ) );
        }

        public void DeleteAll()
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM CountryState; " );
        }

        public IList<State> GetAllByCountryCode( string countryCode, string sortBy, BoolFilter enabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT SortOrder, CountryCode, StateCode, StateName, Enabled " +
                "FROM State " +
                "WHERE CountryCode = @CountryCode " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "Enabled", RepositoryUtilities.ConvertToFlagFilter( enabled ) ) + " " +
                CreateSortByText( sortBy ),
                DataAccess.CreateParameterString( countryCode ) );

            return ConvertTableToList( table );
        }

        #endregion
    }
}
