﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Domain.Tax;
using Vevo.Domain.Marketing;
using Vevo.Domain.Stores;

namespace Vevo.Domain.Orders
{
    public class CartItemProductKit : ICartItem
    {
        #region Private

        private string _cartItemID = Guid.NewGuid().ToString();
        private string _cartID = String.Empty;

        private Product _product;
        private int _quantity;
        private decimal _productCustomPrice;
        private decimal _enteredPrice = 0.0m;
        private bool _isEnterPrice = false;
        private bool _isSubscriptable = false;

        private OptionItemValueCollection _options;
        private ProductKitItemValueCollection _productKitValueCollection;

        private decimal GetProductKitCollectionPrice(WholesaleStatus wholesaleStatus)
        {
            decimal total =0;
            foreach (ProductKitItemValue value in _productKitValueCollection)
            {
                Product product = DataAccessContext.ProductRepository.GetOne( Culture.Null, value.ProductID, Store.RegularStoreID );
                total += product.GetCheckoutPrice( wholesaleStatus ) * value.Quantity;
            }
            return total;
        }

        private string GenerateNameWithProductKit(
            Culture culture,
            Currency currency )
        {
            StringBuilder sb = new StringBuilder();
            foreach (OptionItemValue optionItemValue in Options)
            {
                sb.Append( "<br>" );
                sb.Append( optionItemValue.GetDisplayedName( culture, currency ) );
            }

            if ((!ProductKits.IsNull) && (ProductKits.Count > 0))
            {
                sb.Append( "<br>" );
                sb.Append( "Items:" );
            }

            foreach (ProductKitItemValue value in ProductKits)
            {
                sb.Append( "<br>" );
                sb.Append( "- " + value.GetGroupName(culture,Store.RegularStoreID) + ":" + value.GetDisplayedName(culture,Store.RegularStoreID));
            }

            return sb.ToString();
        }

        private string GenerateRecurringPriceText(
            Currency currency,
            decimal recurringRegularAmount,
            decimal recurringTrialAmount )
        {
            StringBuilder sb = new StringBuilder();

            if (IsRecurring)
            {
                sb.AppendFormat( "<br/>Payment every {0} {1}<br/>",
                    RecurringInterval,
                    GetRecurringUnitText( RecurringIntervalUnit ) );

                sb.Append( GenerateDirectStringTrialAmount(
                    currency, RecurringNumberOfTrialCycles, recurringTrialAmount ) );

                sb.Append( GenerateDirectStringRegularAmount(
                    currency, RecurringNumberOfTrialCycles, RecurringNumberOfCycles, recurringRegularAmount ) );
            }

            return sb.ToString();
        }

        private static string GetRecurringUnitText( string recurringUnit )
        {
            if (recurringUnit == "Day")
                return "day(s)";
            else if (recurringUnit == "Month")
                return "month(s)";
            else if (recurringUnit == "Year")
                return "year(s)";
            else
                return "";
        }


        private string GenerateDirectStringTrialAmount(
            Currency currency,
            int recurringIntervalNumberOfTrialCycle,
            decimal totalRecurringTrialAmount )
        {
            if (recurringIntervalNumberOfTrialCycle == 0)
                return String.Empty;
            else if (recurringIntervalNumberOfTrialCycle == 1)
            {
                if (totalRecurringTrialAmount == 0)
                {
                    return String.Format( "Free Trial for the first payment<br/>" );
                }
                else
                {
                    return String.Format( "First payment: {0} <br/>",
                        currency.FormatPrice( totalRecurringTrialAmount ) );
                }
            }
            else
            {
                if (totalRecurringTrialAmount == 0)
                {
                    return String.Format(
                        "Free Trial for the first {0} payments<br/>",
                        recurringIntervalNumberOfTrialCycle );
                }
                else
                {
                    return String.Format(
                        "First {0} payments: {1} each<br/>",
                        recurringIntervalNumberOfTrialCycle,
                        currency.FormatPrice( totalRecurringTrialAmount ) );
                }
            }
        }

        private string GenerateDirectStringRegularAmount(
            Currency currency,
            int recurringNumberOfTrialCycles,
            int recuringNumberOfCycles,
            decimal totalRecurringRegularAmount )
        {
            if (recurringNumberOfTrialCycles > 0)
            {
                return String.Format( "Next {0} payment(s): {1} each<br/>",
                    recuringNumberOfCycles, currency.FormatPrice( totalRecurringRegularAmount ) );
            }
            else
            {
                return String.Format( "{0} payment(s) of {1}<br/>",
                    recuringNumberOfCycles, currency.FormatPrice( totalRecurringRegularAmount ) );
            }
        }

        private bool ValidateProduct()
        {
            Product product = DataAccessContext.ProductRepository.GetOne( Culture.Null, ProductID, Stores.NullStore.Null.StoreID );
            return !product.IsNull;
        }

        private bool VerifyValidCartOptionItemIDs()
        {
            foreach (OptionItemValue option in Options)
            {
                OptionItem optionItem = DataAccessContext.OptionItemRepository.GetOne( Culture.Null, option.OptionItemID );
                if (optionItem.IsNull)
                    return false;
            }
            return true;
        }

        private bool IsOptionalOptionGroup( OptionGroup.OptionGroupType optionGroupType )
        {
            return optionGroupType == OptionGroup.OptionGroupType.Text
                || optionGroupType == OptionGroup.OptionGroupType.Upload;
        }

        private int HasOptionNotRequired( IList<ProductOptionGroup> productOptionGroups )
        {
            int result = 0;
            foreach (ProductOptionGroup productOptionGroup in productOptionGroups)
            {
                if (IsOptionalOptionGroup( productOptionGroup.OptionGroup.Type ))
                    result++;
            }

            return result;
        }

        private bool ContainsOptionGroupID(
            IList<ProductOptionGroup> productOptionGroups,
            string optionGroupID )
        {
            foreach (ProductOptionGroup productOptionGroup in productOptionGroups)
            {
                if (productOptionGroup.OptionGroupID == optionGroupID)
                    return true;
            }

            return false;
        }

        private bool IsProductContainingSameOptionGroupIDsAsCartItem()
        {
            if (!VerifyValidCartOptionItemIDs())
                return false;

            if (Product.ProductOptionGroups.Count != Options.Count
                && (Product.ProductOptionGroups.Count - HasOptionNotRequired( Product.ProductOptionGroups ))
                    != Options.Count)
            {
                return false;
            }

            foreach (OptionItemValue cartOption in Options)
            {
                if (!ContainsOptionGroupID( Product.ProductOptionGroups, cartOption.OptionItem.OptionGroupID ))
                    return false;
            }

            return true;
        }

        #endregion

        #region Constructors

        public CartItemProductKit(
            string cartID,
            Product product,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKitValueCollection,
            int quantity )
        {
            _cartID = cartID;
            _product = product;
            _options = options;
            _productKitValueCollection = productKitValueCollection;
            _quantity = quantity;
        }

        public CartItemProductKit(
            string cartID,
            Product product,
            OptionItemValueCollection options,
            ProductKitItemValueCollection productKitValueCollection,
            int quantity,
            decimal customUnitPrice )
        {
            _cartID = cartID;
            _product = product;
            _quantity = quantity;
            _options = options;
            _productKitValueCollection = productKitValueCollection;
            _productCustomPrice = customUnitPrice;
        }

        #endregion

        #region Public Properties

        public virtual bool IsNull
        {
            get { return false; }
        }

        public string CartItemID
        {
            get { return _cartItemID; }
        }

        public string CartID
        {
            get { return _cartID; }
        }

        public Product Product
        {
            get { return _product; }
        }

        public string ProductID
        {
            get { return _product.ProductID; }
        }

        public bool IsCustomPrice
        {
            get { return _product.IsCustomPrice; }
        }

        public decimal ProductCustomPrice
        {
            get { return _productCustomPrice; }
        }

        public decimal EnteredPrice
        {
            get { return _enteredPrice; }
            set { _enteredPrice = value; }
        }

        public bool IsEnterPrice
        {
            get { return _isEnterPrice; }
            set { _isEnterPrice = value; }
        }

        public int Quantity
        {
            get
            {
                return _quantity;
            }
            set
            {
                _quantity = value;
            }
        }

        public bool FixedShipping
        {
            get { return _product.FixedShippingCost; }
        }

        public bool IsDownloadable
        {
            get { return _product.IsDownloadable; }
        }

        public bool IsSubscriptionable
        {
            get { return false; }
            set { _isSubscriptable = value; }
        }

        public double UnitWeight
        {
            get
            {
                double total = Product.Weight + _options.GetOptionWeight();
                if (Product.IsDynamicProductKitWeight)
                {
                    foreach (ProductKitItemValue value in _productKitValueCollection)
                    {
                        Product product = DataAccessContext.ProductRepository.GetOne( Culture.Null, value.ProductID, Store.RegularStoreID );
                        total += value.Quantity * product.Weight;
                    }
				}

                return total;
            }
        }

        public string DiscountGroupID
        {
            get { return _product.DiscountGroupID; }
        }

        public string OptionCombinationID
        {
            get
            {
                ProductStock productStock = _product.FindProductStock( _options.GetUseStockOptionItemIDs() );
                if (productStock != null)
                    return productStock.OptionCombinationID;
                else
                    return "0";
            }
        }

        public bool IsGiftCertificate
        {
            get { return false; }
        }

        public CartItemGiftDetails GiftDetails
        {
            get { return CartItemGiftDetails.Null; }
        }

        public decimal GiftPrice
        {
            get { return 0; }
        }

        public OptionItemValueCollection Options
        {
            get { return _options; }
        }

        public bool IsTaxable
        {
            get { return !String.IsNullOrEmpty( TaxClassID ) && TaxClassID != "0"; }
        }

        public string TaxClassID
        {
            get { return _product.TaxClassID; }
        }

        public bool IsRecurring
        {
            get { return _product.IsRecurring; }
        }

        public int RecurringInterval
        {
            get { return _product.IsRecurring ? _product.ProductRecurring.RecurringInterval : 0; }
        }

        public string RecurringIntervalUnit
        {
            get { return _product.IsRecurring ? _product.ProductRecurring.RecurringIntervalUnit : String.Empty; }
        }

        public int RecurringNumberOfCycles
        {
            get { return _product.IsRecurring ? _product.ProductRecurring.RecurringNumberOfCycles : 0; }
        }

        public int RecurringNumberOfTrialCycles
        {
            get { return _product.IsRecurring ? _product.ProductRecurring.RecurringNumberOfTrialCycles : 0; }
        }

        public bool IsPromotion
        {
            get { return false; }
        }

        //public IPromotionSelected PromotionSelected
        //{
        //    get { return PromotionSelected.Null; }
        //}

        public bool IsProductKit
        {
            get { return true; }
        }

        public ProductKitItemValueCollection ProductKits
        {
            get { return _productKitValueCollection; }
        }

        #endregion

        #region Public Methods
        public string GetName( Culture culture, Currency currency )
        {
            return Product.Locales[culture].Name + GenerateNameWithProductKit( culture, currency);
        }

        public string GetNameByStoreID( Culture culture, Currency currency, string storeID )
        {
            return Product.Locales[culture].Name + GenerateNameWithProductKit( culture, currency );
        }

        public string GetOrderItemName(
            Culture culture,
            Currency currency,
            OrderAmountPackage orderAmountPackage )
        {
            return GetName( culture, currency )
                + GenerateRecurringPriceText(
                    currency,
                    orderAmountPackage.RecurringRegularAmount.Total,
                    orderAmountPackage.RecurringTrialAmount.Total );
        }

        public string GetPreCheckoutTooltip(
            Currency currency,
            WholesaleStatus wholesaleStatus,
            out string mainText )
        {
            return GetPreCheckoutTooltip(
                currency,
                wholesaleStatus,
                "[$RecurringTooltipMainText]",
                "[$RecurringProductDescription]",
                "[$RecurringShoppingCartWarning]",
                DataAccessContext.StoreRetriever.GetCurrentStoreID(),
                out mainText );
        }

        public string GetPreCheckoutTooltip(
           Currency currency,
           WholesaleStatus wholesaleStatus,
           string recurringTooltipMainText,
           string recurringProductDescription,
           string recurringShoppingCartWarning,
           string storeID,
           out string mainText )
        {
            if (!IsRecurring)
            {
                mainText = String.Empty;
                return String.Empty;
            }
            else
            {
                mainText = recurringTooltipMainText;

                StringBuilder sb = new StringBuilder();
                sb.AppendLine( recurringProductDescription );
                sb.AppendLine( GenerateRecurringPriceText(
                    currency,
                    GetRecurringRegularUnitPrice( wholesaleStatus, storeID ) * Quantity,
                    GetRecurringTrialUnitPrice( wholesaleStatus ) * Quantity ) );
                sb.AppendLine( recurringShoppingCartWarning );

                return sb.ToString();
            }
        }

        // Pre-checkout unit price includes VAT (if the option "Tax Included" is set)
        public virtual decimal GetShoppingCartUnitPrice( WholesaleStatus wholesaleStatus )
        {
            decimal productPrice = 0;
            if (!IsCustomPrice)
                productPrice = GetUnitPriceWithoutProductOption( wholesaleStatus);
            else
                productPrice = ProductCustomPrice;

            return productPrice + Options.GetOptionPrice( productPrice ); ;
        }

        // Checkout unit price does not include any tax
        public virtual decimal GetCheckoutUnitPrice( WholesaleStatus wholesaleStatus )
        {
            decimal priceWithTax = GetShoppingCartUnitPrice( wholesaleStatus );

            IncludedTaxCalculator includedTaxCalculator =
                new IncludedTaxCalculator( DataAccessContext.Configurations );

            decimal includedTaxAmount = includedTaxCalculator.GetAmount(
                priceWithTax, TaxClassID != "0" );

            return priceWithTax - includedTaxAmount;
        }

        // Pre-checkout unit price includes VAT but exclude option price
        public virtual decimal GetUnitPriceWithoutProductOption( WholesaleStatus wholesaleStatus )
        {
            if (!Product.IsDynamicProductKitPrice)
            {
                return Product.GetCheckoutPrice( wholesaleStatus );
            }
            else
            {
                return Product.GetCheckoutPrice( wholesaleStatus ) + GetProductKitCollectionPrice(wholesaleStatus);
            }
        }

        public virtual decimal GetCheckoutIncludedTaxAmount( WholesaleStatus wholesaleStatus )
        {
            IncludedTaxCalculator includedTaxCalculator =
                new IncludedTaxCalculator( DataAccessContext.Configurations );

            return includedTaxCalculator.GetAmount( GetShoppingCartUnitPrice( wholesaleStatus ), TaxClassID != "0" )
                * Quantity;
        }

        public virtual decimal GetRecurringRegularUnitPrice( WholesaleStatus wholesaleStatus )
        {
            decimal recurringAmount = Product.GetRecurringRegularPrice( wholesaleStatus );
            return recurringAmount;
        }

        public virtual decimal GetRecurringRegularUnitPrice( WholesaleStatus wholesaleStatus, string storeID )
        {
            decimal recurringAmount = Product.GetRecurringRegularPrice( wholesaleStatus, storeID );
            return recurringAmount;
        }

        public virtual decimal GetRecurringTrialUnitPrice( WholesaleStatus wholesaleStatus )
        {
            decimal recurringTrialAmount = Product.GetRecurringTrialPrice( wholesaleStatus );
            return recurringTrialAmount ;
        }

        public virtual decimal GetSubtotal( WholesaleStatus wholesaleStatus )
        {
            return GetCheckoutUnitPrice( wholesaleStatus ) * Quantity;
        }

        public virtual decimal GetSubtotalIncludedTax( WholesaleStatus wholesalestatus )
        {
            return GetShoppingCartUnitPrice( wholesalestatus ) * Quantity;
        }

        public bool IsShippable()
        {
            return (IsGiftCertificate && GiftDetails.NeedPhysical) ||
                    (!IsGiftCertificate && !IsDownloadable && !IsSubscriptionable);
        }

        public bool ValidateReferredObjects()
        {
            if (ValidateProduct()
                && IsProductContainingSameOptionGroupIDsAsCartItem())
                return true;
            else
                return false;
        }

        #endregion
    }
}
